/*
 * SWE Funktionsbibliothek Objektfilter
 * Copyright (C) 2011-2020 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.sys.funclib.objfilter.interpreter;

import java.util.List;

import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Operation, die ermittelt, ob eine Zeichenkette eine andere Zeichenkette
 * enthält oder ermittelt , ob ein Sytemobjekt in einer Menge enthalten ist.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class EnthaeltOperation extends Operation {

    public EnthaeltOperation(final Argument container, Argument element) {
        super(Operator.ENTHAELT, container, element);
    }

    EnthaeltOperation(List<Argument> argumente) {
        super(Operator.ENTHAELT, argumente);
    }

    @Override
    public final Wert auswerten(final SystemObject obj, List<FehlerWert> errors) {
        final Wert arg0 = getArgument(0).auswerten(obj, errors);
        if (arg0 instanceof TextWert) {
            final String text = arg0.getText();
            final String subText = getArgument(1).auswerten(obj, errors).getText();
            if (!subText.isEmpty()) {
                return new BoolWert(text.contains(subText));
            }

            return FehlerWert.fehler(errors, getOperator(), obj, "Der zu prüfende Teiltext darf nicht leer sein");

        } else if (arg0 instanceof MengeWert) {
            final ObjectSet menge = arg0.getMenge();
            final SystemObject element = getArgument(1).auswerten(obj, errors).getSystemObject();

            if (menge == null) {
                return FehlerWert.fehler(errors, getOperator(), obj, "Das erste Argument liefert keine Menge");
            }
            if (element == null) {
                return FehlerWert.fehler(errors, getOperator(), obj,
                        "Das in der Menge zu suchende Element ist nicht gültig");
            }
            return new BoolWert(menge.getElements().contains(element));
        }

        return FehlerWert.fehler(errors, getOperator(), obj, "Das erste Argument ist weder eine Menge noch ein Text");
    }

    @Override
    public final Class<? extends Wert> getErgebnisTyp() {
        return BoolWert.class;
    }

    @Override
    public final boolean verifiziereArgumente(List<VerifizierungsFehler> fehler) {
        if (getArgument(0).getErgebnisTyp() != MengeWert.class && getArgument(0).getErgebnisTyp() != TextWert.class
                && getArgument(0).getErgebnisTyp() != Wert.class) {
            fehler.add(new VerifizierungsFehler(this, "Argument 1 weder eine Menge noch ein Text"));
        }
        if (getArgument(1).getErgebnisTyp() != SystemobjektWert.class
                && getArgument(1).getErgebnisTyp() != TextWert.class && getArgument(0).getErgebnisTyp() != Wert.class) {
            fehler.add(new VerifizierungsFehler(this, "Argument 2 weder ein Systemobjekt noch ein Text"));
        }
        return fehler.isEmpty();
    }

    @Override
    public final boolean checkArgumentTyp(int idx, Class<? extends Wert> clazz) {
        switch (idx) {
        case 0:
            return clazz == TextWert.class || clazz == MengeWert.class || clazz == Wert.class;
        case 1:
            return clazz == SystemobjektWert.class || clazz == TextWert.class || clazz == Wert.class;
        default:
            break;
        }
        return false;
    }
}
